//
// 
//        Flog 0.5
// 
// 
//                                       ani joshi
//                                       Mark Coletti
// 
//



#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include <string>
#include <list>
#include <map>

#ifdef HAVE_BASENAME
#include <libgen.h>
#endif

#include "logline.h"




bool summary = false;           // true if want just summary output
bool bydate  = false;           // true if want reports by date
bool byfile  = false;           // true if want reports by file
bool byemail = false;           // true if want reports by e-mail address
bool fullsum = false;		// true if want full summary output


void
usage( char const * fn )
{

    cerr << fn << ": [-x fn] [-hsVdfe]\n"
         << "\t-x fn\tfilename of xfer log\n"
         << "\t-h\tthis help\n"
         << "\t-s\tsummaries only\n"
         << "\t-V\tprint version\n"
         << "\t-d\tdisplay by date\n"
         << "\t-f\tdisplay by file\n"
         << "\t-e\tdisplay by e-mail address\n"
	 << "\t-S\tdisplay full summary\n";

} // usage()




void
pr_entry_timestamp( Logline const& entry, ostream & os )
{
  os << entry.weekday() << " " 
     << entry.month() << ", " << entry.day() << " " << entry.year() << " "
     << entry.time();
} // pr_entry_timestamp





void
analyze_log( list< Logline > const & log_entries )
{
    map< string, list< Logline > > filelist; // keeps track by filename
    map< string, list< Logline> > emaillist; // keeps track by email address
    map< string, list< Logline> > datelist;  // keeps track by date

    string fastemail;
    string fastip;
    string tempfilename;
    string luckyfile;
    string tempemail;
    string luckyemail;


    double totalxfers = 0;
    double totalsize = 0;
    float maxspeed = 0;

    cout << "Analyzing ... " << flush;


                                // arbitrarily set the best speed to
                                // the first entry to seed it for best
                                // speed calculation

    maxspeed = log_entries.front().speed();


    for ( list< Logline >::const_iterator log_entry = log_entries.begin();
          log_entry != log_entries.end();
          log_entry++ )
      {

        filelist[ log_entry->filename() ].push_back( *log_entry );

        emaillist[ log_entry->email() ].push_back( *log_entry );

        datelist[ log_entry->date() ].push_back( *log_entry );

        totalsize += log_entry->xfersize();

	totalxfers++;

        if ( log_entry->speed() > maxspeed )
          {
            maxspeed  = log_entry->speed();
            fastemail = log_entry->email();
            fastip    = log_entry->ip();
          }

      }
    maxspeed=maxspeed/1024;
    totalsize = totalsize/1024;
    float average_size;
    average_size=(totalsize/totalxfers);
    cout << "done\n\n";


    if(fullsum)
      {
	if(totalsize<1000)
		cout << "Total bytes xfered:\t\t" << totalsize << " kb\n";
	else if(totalsize<1024000){
		totalsize=totalsize/1024;
		cout << "Total bytes xfered:\t\t" << totalsize << " megs\n";
	}
	else{
		totalsize=totalsize/1048576;
		cout << "Total bytes xfered:\t\t" << totalsize << " gigs.\n";
	}	
	cout << "Total files xfered:\t\t" << totalxfers << endl;
	if(average_size<1024){
		cout.setf(ios::fixed,ios::floatfield);
		cout << setprecision(2) << "Average file size:\t" << average_size << " kb\n";
	}
	else{
		average_size=average_size/1024;
		cout.setf(ios::fixed,ios::floatfield);
		cout << setprecision(2) << "Average file size:\t" << average_size << " mb\n";
	}
	cout << "Fastest xfer:   " << maxspeed << " kb/s  set by " << fastemail << endl;
	cout << "Fastest xfer from the machine at:\t" << fastip << endl;
      }
    if ( byemail )
      {

        cout << "hits by e-mail address:\n\n";

        for ( map< string, list< Logline >  >::const_iterator email_entry = 
                emaillist.begin();
              email_entry != emaillist.end();
              email_entry ++ )
          {
            cout << email_entry->first << ":";

            if ( summary )
              {
                cout << "\t";
                cout << email_entry->second.size() << "\n";
              }
            else
              {
                cout << "\n";

                for ( list< Logline >::const_iterator entry = 
                        email_entry->second.begin();
                      entry != email_entry->second.end();
                      entry++ )
                  {
                    cout << "    ";
                    pr_entry_timestamp( *entry, cout );
                    cout << "\t" 
                         << basename( entry->filename().c_str() ) << "\n";
                  }

                cout << "\n";
              }

          }

        cout << "\n";

      }



    if ( byfile )
      {

        cout << "hits by file:\n\n";

        for ( map< string, list< Logline >  >::const_iterator file_entry = 
                filelist.begin();
              file_entry != filelist.end();
              file_entry ++ )
          {
            cout << file_entry->first << ":";

            if ( summary )
              {
                cout << "\t"
                     << file_entry->second.size() 
                     << "\n";
              }
            else
              {
                cout << "\n";
                for ( list< Logline >::const_iterator entry = 
                        file_entry->second.begin();
                      entry != file_entry->second.end();
                      entry++ )
                  {
                    cout << "    ";
                    pr_entry_timestamp( *entry, cout );
                    cout << "\t"<< entry->speed() / 1000 << " kb/sec\t";
                    cout << entry->email() << "\n";
                  }

                cout << "\n";
              }
          }

        cout << "\n";

      }



    if ( bydate )
      {

        cout << "hits by date:\n\n";


        string foo( ' ', 22 );      // silly buggers to get @#$% stuff to
        string bar( ' ', 28 );      // line up properly because @#$% 
        string filename;            // egcs iostreams not properly implemented

        for ( map< string, list< Logline >  >::const_iterator date_entry = 
                datelist.begin();
              date_entry != datelist.end();
              date_entry ++ )
          {

            cout << date_entry->first << ":";

            if ( summary )
              {
                cout << "\t"
                     << date_entry->second.size()
                     << "\n";
              }
            else
              {
                cout << "\n";


                for ( list< Logline >::const_iterator entry = 
                        date_entry->second.begin();
                      entry != date_entry->second.end();
                      entry++ )
                  {
                                // I want to print out the file names
                                // left justified in a 22 character
                                // wide field, followed by the e-mail
                                // address in a 28 character wide,
                                // right justified field.  I spent
                                // _hours_ playing inane games with
                                // egcs' iostreams implementation and
                                // failed to get it to act correctly.
                                // So.  I instead copy filenames and
                                // e-mail addresses over strings of
                                // the right size and by Ghod I get
                                // what I want _THAT_ way.

                    filename = basename( entry->filename().c_str() );

                    copy( filename.begin(), filename.end(), foo.begin() );

                    copy( entry->email().rbegin(), entry->email().rend(), 
                          bar.rbegin() );

                    cout << "    " 
                         << foo << " "
                         << bar << " "
                         << (entry->speed() / 1000) << " kbs\n";

                    foo.assign( ' ', foo.size() ); // reset bogus strings for
                    bar.assign( ' ', bar.size() ); // next loop

                  }

                cout << "\n";
              }
          }

        cout << "\n";

        cout.setf( ios::left, ios::adjustfield );

      }

} // analyze_log




bool
read_logfile( string const& logfilename, list< Logline > & log_entries )
{


    ifstream in;

    in.open( logfilename.c_str() );

    if ( ! in )
    {
      cerr << "\n" << logfilename << " not found or you don't "
           << "have permissions to read the file.\n";

      return false;
    }


    Logline         current_logline;

    cout << "Reading data from " << logfilename << " ... " << flush;

    while ( in >> current_logline )
    {
        if ( in.eof() ) break;

        log_entries.push_back( current_logline );

    }

    cout << "done\n\n";

    return true;

} // read_logfile



int
main( int argc, char *argv[] )
{
    string logfilename( "/var/log/xferlog" );


    extern char *optarg;
    extern int  optind;
    int         ch;

    while ((ch = getopt(argc, argv, "x:hsVdfeS")) != -1)
      switch(ch) {
      case 'h':
        usage( argv[0] );
        exit( 1 );
        break;

      case 's':
        summary = true;
        break;

      case 'S':
	fullsum = true;
	break;

      case 'V':
        cerr << "flog v0.5\n";   // XXX Duh ... need to add more here.
        exit( 0 );
        break;

      case 'd':
        bydate = true;
        break;

      case 'f':
        byfile = true;
        break;

      case 'e':
        byemail = true;
        break;

      case 'x':
        logfilename = optarg;
        break;

      case '?':
      default:
        cerr << "invalid argument " << optarg << "\n";
        usage( argv[0] );
        exit( 1 );
      }

    argc -= optind;
    argv += optind;


    list< Logline > log_entries;

    if ( ! read_logfile( logfilename, log_entries ) )
      {
        exit( 3 );
      }

    analyze_log( log_entries );


    exit( 0 );
}

