//
// logline.cc
//
//
//        Flog 0.5
//                                        ani joshi
//                                        Mark Coletti
//


#include "logline.h"

#include <iostream>
#include <strstream>
#include <iomanip>


static const char * ident_ =
   "$Id: logline.cc,v 1.6 1999/05/08 00:51:33 mcoletti Exp $";



Logline::Logline()
  : day_( 0 ), 
    year_( 0 ), 
    xfertime_( 0 ), 
    xfersize_( 0 ), 
    speed_( 0 ), 
    compressed_( false ), 
    tarred_( false ),
    ascii_( false ),
    outgoing_( false ),
    anonymous_( false ),
    guest_( false ),
    user_( false ),
    rfc931_( false )
{
} // Logline::Logline()



string const & 
Logline::email() const
{
  return email_;
}


string const& 
Logline::filename() const
{
  return file_;
}


string const & 
Logline::ip() const
{
  return ip_;
}


int
Logline::xfersize() const
{
  return xfersize_;
}


int
Logline::xfertime() const
{
  return xfertime_;
}


float 
Logline::speed() const
{
  return speed_;
}


bool
Logline::compressed() const
{
  return compressed_;
} // Logline::compressed



bool
Logline::uncompressed() const
{
  return ! compressed_;
} // Logline::uncompressed


bool
Logline::tarred() const
{
  return tarred_;
} // Logline::tarred


bool
Logline::ascii() const
{
  return ascii_;
} // Logline::ascii


bool
Logline::binary() const
{
  return ! ascii_;
} // Logline::binary


bool
Logline::outgoing() const
{
  return outgoing_;
} // Logline::outgoing


bool
Logline::incoming() const
{
  return ! outgoing_;
} // Logline::incoming


bool
Logline::anonymous() const
{
  return anonymous_;
} // Loglien::anonymous


bool
Logline::guest() const
{
  return guest_;
} // Logline::guest



bool
Logline::user() const
{
  return user_;
} // Logline::user



bool
Logline::rfc931() const
{
  return rfc931_;
} // Logline::rfc931


static
char* months[12] = { "Jan",
                     "Feb",
                     "Mar",
                     "Apr",
                     "May",
                     "Jun",
                     "Jul",
                     "Aug",
                     "Sep",
                     "Oct",
                     "Nov",
                     "Dec" };


// converts months into integers in [0..11]; returns -1 if given invalid
// month string
//
static
int
month_to_int_( string const & month_str )
{
  int i;
  for ( i = 0; i < 12 && months[i] != month_str; i++ );
  
  return ( i == 12 ) ? -1 : i;  // return -1 if we didn't properly map month

} // month_to_int_



string
Logline::date() const
{
  strstream ss;

  ss << setfill('0') << year_ << setw(2) << month_to_int_( month_ ) 
     << setw(2) << day_;

  string tmp_str;

  getline( ss, tmp_str );

  return tmp_str;

} // Logline::date() const



istream&
operator>>( istream & is, Logline& ll )
{
  is >> ll.weekday_ >> ll.month_ >> ll.day_ >> ll.time_;
  is >> ll.year_ >> ll.xfertime_ >> ll.ip_;
  is >> ll.xfersize_ >> ll.file_;

  char c;

  is >> c;

  switch ( c ) 
    {
    case 'a' : ll.ascii_ = true;  break;
    case 'b' : ll.ascii_ = false; break;
    }

  string s;                     // pick out the C, U, T flags

  is >> s;

  for ( string::const_iterator i = s.begin();
        i != s.end();
        i++ )
    {
      if      ( 'C' == *i ) { ll.compressed_ = true; }
      else if ( 'U' == *i ) { ll.compressed_ = false; }
      else if ( 'T' == *i ) { ll.tarred_     = true; }
    }


  is >> c;                      // outgoing or incoming flag

  switch ( c ) 
    {
    case 'o' : ll.outgoing_ = true;  break;
    case 'i' : ll.outgoing_ = false; break;
    }


  is >> c;                      // access mode

  switch ( c ) 
    {
    case 'a' : ll.anonymous_ = true;  break;
    case 'g' : ll.guest_     = true; break;
    case 'r' : ll.user_      = true; break;
    }


  is >> ll.email_ >> ll.service_;


  int authentication_method;

  is >> authentication_method;

  if ( 1 == authentication_method )
    {
      ll.rfc931_ = true;
    }


  is >> ll.userid_;             // '*' if anonymous

  char ch,ch2;
  ch=is.peek();
  if(ch!='\n')
	is >> ch2;

  ll.speed_ = ( ll.xfersize_ / ll.xfertime_ );

  return is;

} // operator>>





ostream&
operator<<( ostream & os, Logline const& ll )
{
  os << ll.file_ << " " ;

  os << ll.weekday_ << " " << ll.month_ << " "  << ll.day_ << " " << ll.year_;
  os << ll.time_ << "\n";

  os << ll.xfertime_ << " " 
     << ll.xfersize_ << "\n";

  os << ll.email_ << " " << ll.ip_ << " " << ll.service_;

  os << "\n\n";

  return os;
} // operator<<
